@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2003-2004  Narcis Ilisei <inarcis2002@hotpop.com>
  * Copyright (C) 2006       Steve Horbachuk
- * Copyright (C) 2010-2017  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2010-2021  Joachim Wiberg <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
 
 /*
  * For details, see the excellent support page at easyDNS
- * http://support.easydns.com/tutorials/dynamicUpdateSpecs.php
+ * https://fusion.easydns.com/Knowledgebase/Article/View/102/0/dynamic-dns
  */
 #define EASYDNS_UPDATE_IP_REQUEST					\
 	"GET %s?"							\
@@ -50,8 +50,8 @@ static ddns_system_t plugin = {
 	.checkip_url  = DYNDNS_MY_CHECKIP_URL,
 	.checkip_ssl  = DYNDNS_MY_IP_SSL,
 
-	.server_name  = "members.easydns.com",
-	.server_url   = "/dyn/dyndns.php"
+	.server_name  = "api.cp.easydns.com",
+	.server_url   = "/dyn/generic.php"
 };
 
 static int request(ddns_t *ctx, ddns_info_t *info, ddns_alias_t *alias)
@@ -74,6 +74,9 @@ static int response(http_trans_t *trans, ddns_info_t *info, ddns_alias_t *alias)
 {
 	char *resp = trans->rsp_body;
 
+	(void)info;
+	(void)alias;
+
 	DO(http_status_valid(trans->status));
 
 	if (strstr(resp, "NOERROR"))
