@@ -1,6 +1,6 @@
 /* libConfuse interface to parse inadyn.conf v2 format
  *
- * Copyright (C) 2014-2020  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2014-2021  Joachim Wiberg <troglobit@gmail.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -59,6 +59,7 @@ static void conf_errfunc(cfg_t *cfg, const char *format, va_list args)
 	vlogit(LOG_ERR, fmt, args);
 }
 
+#ifdef DROP_CHECK_CONFIG
 /*
  * Convert deprecated 'alias' setting to new 'hostname',
  * same functionality with new name.
@@ -189,6 +190,7 @@ static int validate_custom(cfg_t *cfg, cfg_opt_t *opt)
 
 	return validate_common(cfg, "custom", 1);
 }
+#endif
 
 /* server:port => server:80 if port is not given */
 static int getserver(const char *server, ddns_name_t *name)
@@ -309,9 +311,9 @@ static int cfg_parseproxy(cfg_t *cfg, char *server, tcp_proxy_type_t *type, ddns
 
 static int set_provider_opts(cfg_t *cfg, ddns_info_t *info, int custom)
 {
-	size_t j;
-	const char *str;
 	ddns_system_t *system;
+	const char *str;
+	size_t j;
 
 	if (custom)
 		str = "custom";
@@ -343,6 +345,8 @@ static int set_provider_opts(cfg_t *cfg, ddns_info_t *info, int custom)
 	strlcpy(info->server_url, system->server_url, sizeof(info->server_url));
 
 	info->wildcard = cfg_getbool(cfg, "wildcard");
+	info->ttl = cfg_getint(cfg, "ttl");
+	info->proxied = cfg_getbool(cfg, "proxied");
 	info->ssl_enabled = cfg_getbool(cfg, "ssl");
 	str = cfg_getstr(cfg, "username");
 	if (str && strlen(str) <= sizeof(info->creds.username))
@@ -437,6 +441,12 @@ static int set_provider_opts(cfg_t *cfg, ddns_info_t *info, int custom)
 		else
 			strlcpy(info->checkip_url, "/", sizeof(info->checkip_url));
 
+		if (!strcasecmp(info->checkip_name.name, "default")) {
+			strlcpy(info->checkip_name.name, DDNS_MY_IP_SERVER, sizeof(info->checkip_name.name));
+			strlcpy(info->checkip_url, "/", sizeof(info->checkip_url));
+			/* let local checkip-ssl decide HTTP/HTTPS for ipify */
+		}
+
 		/*
 		 * If a custom checkip server is defined, the
 		 * checkip-ssl setting is fully honored.
@@ -525,12 +535,15 @@ cfg_t *conf_parse_file(char *file, ddns_t *ctx)
 	int ret = 0;
 	size_t i;
 	cfg_opt_t provider_opts[] = {
+		CFG_FUNC    ("include",      &cfg_include),
 		CFG_STR     ("username",     NULL, CFGF_NONE),
 		CFG_STR     ("password",     NULL, CFGF_NONE),
 		CFG_STR_LIST("hostname",     NULL, CFGF_NONE),
 		CFG_STR_LIST("alias",        NULL, CFGF_DEPRECATED),
 		CFG_BOOL    ("ssl",          cfg_true, CFGF_NONE),
 		CFG_BOOL    ("wildcard",     cfg_false, CFGF_NONE),
+		CFG_INT     ("ttl",          -1, CFGF_NODEFAULT),
+		CFG_BOOL    ("proxied",      cfg_false, CFGF_NONE),
 		CFG_STR     ("checkip-server", NULL, CFGF_NONE), /* Syntax:  name:port */
 		CFG_STR     ("checkip-path",   NULL, CFGF_NONE), /* Default: "/" */
 		CFG_BOOL    ("checkip-ssl",    cfg_true, CFGF_NONE),
@@ -541,12 +554,15 @@ cfg_t *conf_parse_file(char *file, ddns_t *ctx)
 	};
 	cfg_opt_t custom_opts[] = {
 		/* Same as a general provider */
+		CFG_FUNC    ("include",      &cfg_include),
 		CFG_STR     ("username",     NULL, CFGF_NONE),
 		CFG_STR     ("password",     NULL, CFGF_NONE),
 		CFG_STR_LIST("hostname",     NULL, CFGF_NONE),
 		CFG_STR_LIST("alias",        NULL, CFGF_DEPRECATED),
 		CFG_BOOL    ("ssl",          cfg_true, CFGF_NONE),
 		CFG_BOOL    ("wildcard",     cfg_false, CFGF_NONE),
+		CFG_INT     ("ttl",          -1, CFGF_NODEFAULT),
+		CFG_BOOL    ("proxied",      cfg_false, CFGF_NONE),
 		CFG_STR     ("checkip-server", NULL, CFGF_NONE), /* Syntax:  name:port */
 		CFG_STR     ("checkip-path",   NULL, CFGF_NONE), /* Default: "/" */
 		CFG_BOOL    ("checkip-ssl",    cfg_true, CFGF_NONE),
@@ -588,10 +604,12 @@ cfg_t *conf_parse_file(char *file, ddns_t *ctx)
 	/* Custom logging, rather than default Confuse stderr logging */
 	cfg_set_error_function(cfg, conf_errfunc);
 
+#ifdef DROP_CHECK_CONFIG
 	/* Validators */
 	cfg_set_validate_func(cfg, "period", validate_period);
 	cfg_set_validate_func(cfg, "provider", validate_provider);
 	cfg_set_validate_func(cfg, "custom", validate_custom);
+#endif
 
 	switch (cfg_parse(cfg, file)) {
 	case CFG_FILE_ERROR:
