@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2003-2004  Narcis Ilisei <inarcis2002@hotpop.com>
  * Copyright (C) 2006       Steve Horbachuk
- * Copyright (C) 2010-2020  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2010-2021  Joachim Wiberg <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -168,6 +168,35 @@ static int server_transaction(ddns_t *ctx, ddns_info_t *provider)
  */
 static int is_address_valid(int family, const char *host)
 {
+	/*
+	 * cloudflare would return requested hostname before client's ip address
+	 * block cloudflare ips so that https://1.1.1.1/cdn-cgi/trace would work
+	 * even if 1.1.1.1 is the first ip in the response body
+	 */
+	static const char *except[] = {
+		"1.1.1.1",
+		"1.0.0.1",
+		"2606:4700:4700::1111",
+		"2606:4700:4700::1001",
+		"1.1.1.2",
+		"1.0.0.2",
+		"2606:4700:4700::1112",
+		"2606:4700:4700::1002",
+		"1.1.1.3",
+		"1.0.0.3",
+		"2606:4700:4700::1113",
+		"2606:4700:4700::1003",
+		"2606:4700:4700::64",
+		"2606:4700:4700::6400"
+	};
+	size_t i;
+
+	for (i = 0; i < NELEMS(except); i++) {
+		if (!strncmp(host, except[i], strlen(host))) {
+			return 0;
+		}
+	}
+
 	if (!verify_addr) {
 		logit(LOG_DEBUG, "IP address validation disabled, %s is thus valid.", host);
 		return 1;
@@ -219,7 +248,7 @@ error:
 static int parse_ipv4_address(char *buffer, char *address, size_t len)
 {
 	int found = 0;
-	char *accept = "0123456789.";
+	static const char *accept = "0123456789.";
 	char *needle, *haystack, *end;
 	struct in_addr  addr;
 
@@ -259,7 +288,7 @@ static int parse_ipv4_address(char *buffer, char *address, size_t len)
 static int parse_ipv6_address(char *buffer, char *address, size_t len)
 {
 	int found = 0;
-	char *accept = "0123456789abcdefABCDEF:";
+	static const char *accept = "0123456789abcdefABCDEF:";
 	char *needle, *haystack, *end;
 	struct in6_addr addr;
 
@@ -431,6 +460,10 @@ static int get_address_iface(ddns_t *ctx, const char *ifname, char *address, siz
 
 static int get_address_backend(ddns_t *ctx, ddns_info_t *info, char *address, size_t len)
 {
+	char name[sizeof(info->checkip_name.name)];
+	char url[sizeof(info->checkip_url)];
+	int rc;
+
 	logit(LOG_DEBUG, "Get address for %s", info->system->name);
 	memset(address, 0, len);
 
@@ -443,6 +476,36 @@ static int get_address_backend(ddns_t *ctx, ddns_info_t *info, char *address, si
 	if (!get_address_remote(ctx, info, address, len))
 		return 0;
 
+	logit(LOG_WARNING, "Communication with checkip server %s failed, "
+	      "run again with 'inadyn -l debug' if problem persists",
+	      info->checkip_name.name);
+
+	/* Skip fallback if it's the same server ... option: flip SSL bit? */
+	if (strstr(info->checkip_name.name, DDNS_MY_IP_SERVER))
+		goto error;
+
+	logit(LOG_WARNING, "Retrying with built-in 'default', api.ipify.org ...");
+
+	/* keep backup, for now, future extension: count failures and replace permanently */
+	strlcpy(name, info->checkip_name.name, sizeof(name));
+	strlcpy(url, info->checkip_url, sizeof(url));
+
+	/* Retry with http(s)://api.ipify.org/ */
+	strlcpy(info->checkip_name.name, DDNS_MY_IP_SERVER, sizeof(info->checkip_name.name));
+	strlcpy(info->checkip_url, "/", sizeof(info->checkip_url));
+	rc = get_address_remote(ctx, info, address, len);
+
+	/* restore backup, for now, the official server may just have temporary problems */
+	strlcpy(info->checkip_name.name, name, sizeof(info->checkip_name.name));
+	strlcpy(info->checkip_url, url, sizeof(info->checkip_url));
+
+	if (!rc) {
+		logit(LOG_WARNING, "Please note, %s seems unstable, consider overriding it "
+		      "in your configuration with 'checkip-server = default'", info->checkip_name.name);
+		return 0;
+	}
+
+error:
 	logit(LOG_ERR, "Failed to get IP address for %s, giving up!", info->system->name);
 
 	return 1;
@@ -516,7 +579,7 @@ static int check_alias_update_table(ddns_t *ctx)
 			ddns_alias_t *alias = &info->alias[i];
 
 /* XXX: TODO time_to_check() will return false positive if the cache
- *     file is missing => causing unnnecessary update.  We should save
+ *     file is missing => causing unnecessary update.  We should save
  *     the cache file with the current IP instead and fall back to
  *     standard update interval!
  */
@@ -570,14 +633,13 @@ static int send_update(ddns_t *ctx, ddns_info_t *info, ddns_alias_t *alias, int
 		goto exit;
 	}
 
+	ctx->request_buf[trans.req_len] = 0;
+	logit(LOG_DEBUG, "Sending alias table update to DDNS server: %s", ctx->request_buf);
+
 #ifdef ENABLE_SIMULATION
 	logit(LOG_WARNING, "In simulation, skipping update to server ...");
 	goto exit;
 #endif
-
-	ctx->request_buf[trans.req_len] = 0;
-	logit(LOG_DEBUG, "Sending alias table update to DDNS server: %s", ctx->request_buf);
-
 	rc = http_transaction(client, &trans);
 	if (rc) {
 		/* Update failed, force update again in ctx->cmd_check_period seconds */
@@ -653,22 +715,29 @@ static int update_alias_table(ddns_t *ctx)
 
 		for (i = 0; i < info->alias_count; i++) {
 			ddns_alias_t *alias = &info->alias[i];
+			char *event = "update";
+			rc = 0;
 
-			if (!alias->update_required)
-				continue;
+			if (!alias->update_required) {
+				if (exec_mode == EXEC_MODE_COMPAT)
+					continue;
+				event = "nochg";
+			} else if ((rc = send_update(ctx, info, alias, &anychange))) {
+				if (exec_mode == EXEC_MODE_COMPAT)
+					break;
+				event = "error";
+			} else {
+				/* Only reset if send_update() succeeds. */
+				alias->update_required = 0;
+				alias->last_update = time(NULL);
 
-			TRY(send_update(ctx, info, alias, &anychange));
-
-			/* Only reset if send_update() succeeds. */
-			alias->update_required = 0;
-			alias->last_update = time(NULL);
-
-			/* Update cache file for this entry */
-			write_cache_file(alias);
+				/* Update cache file for this entry */
+				write_cache_file(alias);
+			}
 
 			/* Run command or script on successful update. */
 			if (script_exec)
-				os_shell_execute(script_exec, alias->address, alias->name);
+				os_shell_execute(script_exec, alias->address, alias->name, event, rc);
 		}
 
 		if (RC_DDNS_RSP_NOTOK == rc || RC_DDNS_RSP_AUTH_FAIL == rc)
@@ -734,7 +803,7 @@ static int get_encoded_user_passwd(void)
 			break;
 		}
 
-		logit(LOG_DEBUG, "Base64 encoded string: %s", encode);
+//		logit(LOG_DEBUG, "Base64 encoded string: %s", encode);
 		info->creds.encoded_password = encode;
 		info->creds.encoded = 1;
 		info->creds.size = strlen(info->creds.encoded_password);
