@@ -1,7 +1,7 @@
 /* Interface for TCP functions
  *
  * Copyright (C) 2003-2004  Narcis Ilisei <inarcis2002@hotpop.com>
- * Copyright (C) 2010-2017  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2010-2021  Joachim Wiberg <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,18 +20,19 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#include <arpa/nameser.h>
 #include <errno.h>
-#include <net/if.h>
-#include <netinet/in.h>
 #include <poll.h>
-#include <resolv.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
+#include <sys/socket.h>
+#include <arpa/nameser.h>
+#include <net/if.h>
+#include <netinet/in.h>
+#include <resolv.h>
 
 #include "log.h"
 #include "tcp.h"
@@ -130,8 +131,9 @@ int tcp_init(tcp_sock_t *tcp, char *msg)
 
 		/* Obtain address(es) matching host/port */
 		memset(&hints, 0, sizeof(struct addrinfo));
-		hints.ai_family = AF_INET;	/* Use AF_UNSPEC to allow IPv4 or IPv6 */
-		hints.ai_socktype = SOCK_DGRAM;	/* Datagram socket */
+		hints.ai_family = AF_UNSPEC;		/* Allow IPv4 or IPv6 */
+		hints.ai_socktype = SOCK_STREAM;	/* Stream socket */
+		hints.ai_flags = AI_NUMERICSERV;	/* No service name lookup */
 		snprintf(port, sizeof(port), "%d", tcp->port);
 
 		s = getaddrinfo(tcp->remote_host, port, &hints, &servinfo);
@@ -143,17 +145,13 @@ int tcp_init(tcp_sock_t *tcp, char *msg)
 		ai = servinfo;
 
 		while (1) {
-			sd = socket(AF_INET, SOCK_STREAM, 0);
+			sd = socket(ai->ai_family, SOCK_STREAM, 0);
 			if (sd == -1) {
 				logit(LOG_ERR, "Error creating client socket: %s", strerror(errno));
 				rc = RC_TCP_SOCKET_CREATE_ERROR;
 				break;
 			}
 
-			set_timeouts(sd, tcp->timeout);
-			tcp->socket = sd;
-			tcp->initialized = 1;
-
 			/* Now we try connecting to the server, on connect fail, try next DNS record */
 			sa  = ai->ai_addr;
 			len = ai->ai_addrlen;
@@ -161,25 +159,28 @@ int tcp_init(tcp_sock_t *tcp, char *msg)
 			if (getnameinfo(sa, len, host, sizeof(host), NULL, 0, NI_NUMERICHOST))
 				goto next;
 
-			logit(LOG_INFO, "%s, %sconnecting to %s(%s:%d)", msg, tries ? "re" : "",
+			set_timeouts(sd, tcp->timeout);
+
+			logit(LOG_INFO, "%s, %sconnecting to %s([%s]:%d)", msg, tries ? "re" : "",
 			      tcp->remote_host, host, tcp->port);
-			if (connect(sd, sa, len)) {
+			if (connect(sd, sa, len) && check_error(sd, tcp->timeout)) {
+			next:
 				tries++;
 
-				if (!check_error(sd, tcp->timeout))
-					break; /* OK */
-			next:
 				ai = ai->ai_next;
 				if (ai) {
 					logit(LOG_INFO, "Failed connecting to that server: %s",
 					      errno != EINPROGRESS ? strerror(errno) : "retrying ...");
-
 					close(sd);
 					continue;
 				}
 
 				logit(LOG_WARNING, "Failed connecting to %s: %s", tcp->remote_host, strerror(errno));
+				close(sd);
 				rc = RC_TCP_CONNECT_FAILED;
+			} else {
+				tcp->socket = sd;
+				tcp->initialized = 1;
 			}
 
 			break;
@@ -220,8 +221,12 @@ int tcp_send(tcp_sock_t *tcp, const char *buf, int len)
 
 	if (!tcp->initialized)
 		return RC_TCP_OBJECT_NOT_INITIALIZED;
-
+again:
 	if (send(tcp->socket, buf, len, 0) == -1) {
+		int err = errno;
+		if(err == EAGAIN || err == EWOULDBLOCK){
+			goto again;
+		}
 		logit(LOG_WARNING, "Network error while sending query/update: %s", strerror(errno));
 		return RC_TCP_SEND_ERROR;
 	}
@@ -244,11 +249,16 @@ int tcp_recv(tcp_sock_t *tcp, char *buf, int len, int *recv_len)
 
 	while (remaining_bytes > 0) {
 		int bytes;
+		int err = 0;
 		int chunk_size = remaining_bytes > TCP_DEFAULT_READ_CHUNK_SIZE
 			? TCP_DEFAULT_READ_CHUNK_SIZE
 			: remaining_bytes;
 
 		bytes = recv(tcp->socket, buf + total_bytes, chunk_size, 0);
+		err = errno;
+		if(bytes == -1 && (err == EAGAIN || err == EWOULDBLOCK)){
+			continue;
+		}
 		if (bytes < 0) {
 			logit(LOG_WARNING, "Network error while waiting for reply: %s", strerror(errno));
 			rc = RC_TCP_RECV_ERROR;
